"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UndClient = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _crypto = _interopRequireDefault(require("crypto"));

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _proto = _interopRequireDefault(require("../messages/proto"));

var _request = _interopRequireDefault(require("request"));

var _secp256k = _interopRequireDefault(require("secp256k1"));

var _config = _interopRequireDefault(require("../config"));

var fundCrypto = _interopRequireWildcard(require("../crypto"));

var _validateHelper = require("../utils/validateHelper");

var _utils = require("../utils");

var msgFactory = _interopRequireWildcard(require("../messages/factory"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var UndClient = /*#__PURE__*/function () {
  function UndClient(url) {
    var broadcastMode = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "BROADCAST_MODE_SYNC";
    (0, _classCallCheck2["default"])(this, UndClient);

    if (!url) {
      throw new Error("FUND Mainchain API url should not be null");
    }

    this.url = url;
    this.chainId = null;
    this.path = "".concat(_config["default"].HD_PATH, "0");
    this.bech32MainPrefix = _config["default"].BECH32_PREFIX;
    this.broadcastMode = broadcastMode;
    this.accountNumber = -1;
    this.address = null;
    this.privKey = null;
    this.pubKeyAny = null;
  }
  /*
   * Class setters and intialiser
   */

  /**
   * Initialize the client with the chain's ID. Asynchronous.
   * @return {Promise}
   */


  (0, _createClass2["default"])(UndClient, [{
    key: "initChain",
    value: function () {
      var _initChain = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
        var data;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (this.chainId) {
                  _context.next = 7;
                  break;
                }

                _context.next = 3;
                return this.getChainInfo();

              case 3:
                data = _context.sent;
                this.chainId = data.node_info.network;
                this.node_info = data.node_info;
                this.node_app_version = data.application_version;

              case 7:
                return _context.abrupt("return", this);

              case 8:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function initChain() {
        return _initChain.apply(this, arguments);
      }

      return initChain;
    }()
    /**
     * Set the private key for the client
     * @param {String|Buffer} privKey private key as a hex string or Buffer
     */

  }, {
    key: "setPrivateKey",
    value: function () {
      var _setPrivateKey = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(privKey) {
        var accData;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                this.privKey = Buffer.from(privKey, "hex");
                this.pubKeyAny = msgFactory.getPubKeyAny(this.privKey);
                this.address = fundCrypto.getAddressFromPrivateKey(this.privKey.toString("hex"), this.bech32MainPrefix);
                _context2.next = 5;
                return this.getAccount(this.address);

              case 5:
                accData = _context2.sent;

                if (accData !== null && accData !== void 0 && accData.account) {
                  this.accountNumber = accData.account.account_number;
                }

              case 7:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function setPrivateKey(_x) {
        return _setPrivateKey.apply(this, arguments);
      }

      return setPrivateKey;
    }()
  }, {
    key: "clean",
    value: function clean() {
      this.accountNumber = -1;
      this.address = null;
      this.privKey = null;
      this.pubKeyAny = null;
    }
  }, {
    key: "getPubKeyAny",
    value: function getPubKeyAny() {
      return this.pubKeyAny;
    }
    /**
     * Set the mode for broadcasting a Tx
     * "BROADCAST_MODE_UNSPECIFIED", "BROADCAST_MODE_BLOCK", "BROADCAST_MODE_SYNC", "BROADCAST_MODE_ASYNC"
     * @param {String} broadcastMode sync = wait for checkTx, async = send and forget (faster but less guarantees), block = wait for block to process (default sync)
     */

  }, {
    key: "setBroadcastMode",
    value: function setBroadcastMode(broadcastMode) {
      this.broadcastMode = broadcastMode;
    }
    /*
     * Send Transaction functions
     */

    /**
     * Transfer FUND to an address
     * @param {String} toAddress
     * @param {Number} amount
     * @param {Object} fee
     * @param {String} denom optional denom
     * @param {String} fromAddress optional fromAddress
     * @param {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "transferUnd",
    value: function () {
      var _transferUnd = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(toAddress, amount, fee) {
        var denom,
            fromAddress,
            memo,
            _convertAmount,
            amt,
            dnm,
            _convertAmount2,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args3 = arguments;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                denom = _args3.length > 3 && _args3[3] !== undefined ? _args3[3] : "nund";
                fromAddress = _args3.length > 4 && _args3[4] !== undefined ? _args3[4] : this.address;
                memo = _args3.length > 5 && _args3[5] !== undefined ? _args3[5] : "";

                if (fromAddress) {
                  _context3.next = 5;
                  break;
                }

                throw new Error("fromAddress should not be empty");

              case 5:
                if (toAddress) {
                  _context3.next = 7;
                  break;
                }

                throw new Error("toAddress should not be empty");

              case 7:
                if (!(amount === 0)) {
                  _context3.next = 9;
                  break;
                }

                throw new Error("amount should not be zero");

              case 9:
                if (fundCrypto.checkAddress(toAddress, this.bech32MainPrefix)) {
                  _context3.next = 11;
                  break;
                }

                throw new Error("invalid toAddress");

              case 11:
                if (fundCrypto.checkAddress(fromAddress, this.bech32MainPrefix)) {
                  _context3.next = 13;
                  break;
                }

                throw new Error("invalid fromAddress");

              case 13:
                (0, _validateHelper.checkNumber)(amount, "amount");
                _convertAmount = (0, _utils.convertAmount)(amount, denom), amt = _convertAmount.amt, dnm = _convertAmount.dnm;
                _convertAmount2 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount2.amt, feeDnm = _convertAmount2.dnm;
                _context3.next = 18;
                return this.getAccount(fromAddress);

              case 18:
                accData = _context3.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context3.next = 21;
                  break;
                }

                throw new Error(accData.message);

              case 21:
                // ---------------------------------- (1)txBody ----------------------------------
                txBody = msgFactory.getMsgSendTx(fromAddress, toAddress, amt, dnm, memo); // --------------------------------- (2)authInfo ---------------------------------

                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context3.abrupt("return", this.broadcast(signedTxBytes));

              case 25:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function transferUnd(_x2, _x3, _x4) {
        return _transferUnd.apply(this, arguments);
      }

      return transferUnd;
    }()
    /**
     * Register a BEACON
     * @param moniker {String} moniker
     * @param name {String} name optional name
     * @param fromAddress {String} fromAddress
     * @param gas {Number} gas optional gas
     * @param memo {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "registerBeacon",
    value: function () {
      var _registerBeacon = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(moniker) {
        var name,
            fromAddress,
            gas,
            memo,
            accData,
            beaconParams,
            txBody,
            authInfo,
            signedTxBytes,
            _args4 = arguments;
        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                name = _args4.length > 1 && _args4[1] !== undefined ? _args4[1] : "";
                fromAddress = _args4.length > 2 && _args4[2] !== undefined ? _args4[2] : this.address;
                gas = _args4.length > 3 && _args4[3] !== undefined ? _args4[3] : 100000;
                memo = _args4.length > 4 && _args4[4] !== undefined ? _args4[4] : "";

                if (fromAddress) {
                  _context4.next = 6;
                  break;
                }

                throw new Error("fromAddress should not be empty");

              case 6:
                if (fundCrypto.checkAddress(fromAddress, _config["default"].BECH32_PREFIX)) {
                  _context4.next = 8;
                  break;
                }

                throw new Error("invalid fromAddress");

              case 8:
                if (!(moniker === "" || moniker === null || moniker === undefined)) {
                  _context4.next = 10;
                  break;
                }

                throw new Error("beacon must have a moniker");

              case 10:
                _context4.next = 12;
                return this.getAccount(fromAddress);

              case 12:
                accData = _context4.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context4.next = 15;
                  break;
                }

                throw new Error(accData.message);

              case 15:
                _context4.next = 17;
                return this.getBeaconParams();

              case 17:
                beaconParams = _context4.sent;
                txBody = msgFactory.getMsgRegisterBeacon(moniker, name, fromAddress);
                authInfo = this.compileAuthInfo(accData.account.sequence, beaconParams.params.fee_register, beaconParams.params.denom, gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context4.abrupt("return", this.broadcast(signedTxBytes));

              case 22:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function registerBeacon(_x5) {
        return _registerBeacon.apply(this, arguments);
      }

      return registerBeacon;
    }()
    /**
     * Submit a BEACON timestamp
     * @param beaconId {Number} beacon_id
     * @param hash {String} hash
     * @param submitTime {Number} submit_time
     * @param fromAddress {String} fromAddress
     * @param gas {Number} gas optional gas
     * @param memo {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "recordBeaconTimestamp",
    value: function () {
      var _recordBeaconTimestamp = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(beaconId, hash, submitTime) {
        var fromAddress,
            gas,
            memo,
            accData,
            beaconParams,
            txBody,
            authInfo,
            signedTxBytes,
            _args5 = arguments;
        return _regenerator["default"].wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                fromAddress = _args5.length > 3 && _args5[3] !== undefined ? _args5[3] : this.address;
                gas = _args5.length > 4 && _args5[4] !== undefined ? _args5[4] : 100000;
                memo = _args5.length > 5 && _args5[5] !== undefined ? _args5[5] : "";

                if (fromAddress) {
                  _context5.next = 5;
                  break;
                }

                throw new Error("fromAddress should not be empty");

              case 5:
                if (fundCrypto.checkAddress(fromAddress, _config["default"].BECH32_PREFIX)) {
                  _context5.next = 7;
                  break;
                }

                throw new Error("invalid fromAddress");

              case 7:
                if (!(parseInt(beaconId) <= 0)) {
                  _context5.next = 9;
                  break;
                }

                throw new Error("must have beacon id");

              case 9:
                if (!(hash === "" || hash === null || hash === undefined)) {
                  _context5.next = 11;
                  break;
                }

                throw new Error("beacon must have hash");

              case 11:
                if (!(parseInt(submitTime, 10) <= 0)) {
                  _context5.next = 13;
                  break;
                }

                throw new Error("must have submit time");

              case 13:
                _context5.next = 15;
                return this.getAccount(fromAddress);

              case 15:
                accData = _context5.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context5.next = 18;
                  break;
                }

                throw new Error(accData.message);

              case 18:
                _context5.next = 20;
                return this.getBeaconParams();

              case 20:
                beaconParams = _context5.sent;
                txBody = msgFactory.getMsgRecordBeaconTimestamp(beaconId, hash, submitTime, fromAddress);
                authInfo = this.compileAuthInfo(accData.account.sequence, beaconParams.params.fee_record, beaconParams.params.denom, gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context5.abrupt("return", this.broadcast(signedTxBytes));

              case 25:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function recordBeaconTimestamp(_x6, _x7, _x8) {
        return _recordBeaconTimestamp.apply(this, arguments);
      }

      return recordBeaconTimestamp;
    }()
    /**
     * Register a WRKChain
     * @param moniker {String} moniker
     * @param baseType {String} base_type optional base_type
     * @param name {String} name optional name
     * @param genesis {String} genesis optional genesis
     * @param fromAddress {String} fromAddress
     * @param gas {Number} gas optional gas
     * @param memo {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "registerWRKChain",
    value: function () {
      var _registerWRKChain = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(moniker, baseType) {
        var name,
            genesis,
            fromAddress,
            gas,
            memo,
            accData,
            params,
            txBody,
            authInfo,
            signedTxBytes,
            _args6 = arguments;
        return _regenerator["default"].wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                name = _args6.length > 2 && _args6[2] !== undefined ? _args6[2] : "";
                genesis = _args6.length > 3 && _args6[3] !== undefined ? _args6[3] : "";
                fromAddress = _args6.length > 4 && _args6[4] !== undefined ? _args6[4] : this.address;
                gas = _args6.length > 5 && _args6[5] !== undefined ? _args6[5] : 100000;
                memo = _args6.length > 6 && _args6[6] !== undefined ? _args6[6] : "";

                if (fromAddress) {
                  _context6.next = 7;
                  break;
                }

                throw new Error("fromAddress should not be empty");

              case 7:
                if (fundCrypto.checkAddress(fromAddress, _config["default"].BECH32_PREFIX)) {
                  _context6.next = 9;
                  break;
                }

                throw new Error("invalid fromAddress");

              case 9:
                if (!(moniker === "" || moniker === null || moniker === undefined)) {
                  _context6.next = 11;
                  break;
                }

                throw new Error("wrkchain must have a moniker");

              case 11:
                if (!(baseType === "" || baseType === null || baseType === undefined)) {
                  _context6.next = 13;
                  break;
                }

                throw new Error("wrkchain must have a type");

              case 13:
                _context6.next = 15;
                return this.getAccount(fromAddress);

              case 15:
                accData = _context6.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context6.next = 18;
                  break;
                }

                throw new Error(accData.message);

              case 18:
                _context6.next = 20;
                return this.getWRKChainParams();

              case 20:
                params = _context6.sent;
                txBody = msgFactory.getMsgRegisterWrkChain(moniker, name, genesis, baseType, fromAddress);
                authInfo = this.compileAuthInfo(accData.account.sequence, params.params.fee_register, params.params.denom, gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context6.abrupt("return", this.broadcast(signedTxBytes));

              case 25:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function registerWRKChain(_x9, _x10) {
        return _registerWRKChain.apply(this, arguments);
      }

      return registerWRKChain;
    }()
    /**
     * Submit WRKChain block header hashes
     * @param wrkchainId {Number} wrkchain_id
     * @param height {String} height
     * @param blockHash {String} blockhash
     * @param parentHash {String} parenthash optional parenthash
     * @param hash1 {String} hash1 optional hash1
     * @param hash2 {String} hash2 optional hash2
     * @param hash3 {String} hash3 optional hash3
     * @param fromAddress {String} fromAddress
     * @param gas {Number} gas optional gas
     * @param memo {String} memo optional memo
     * @param sequence {Number} sequence optional sequence
     * @returns {Promise<*>}
     */

  }, {
    key: "recordWRKChainBlock",
    value: function () {
      var _recordWRKChainBlock = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(wrkchainId, height, blockHash, parentHash, hash1, hash2, hash3) {
        var fromAddress,
            gas,
            memo,
            sequence,
            accData,
            params,
            txBody,
            authInfo,
            signedTxBytes,
            _args7 = arguments;
        return _regenerator["default"].wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                fromAddress = _args7.length > 7 && _args7[7] !== undefined ? _args7[7] : this.address;
                gas = _args7.length > 8 && _args7[8] !== undefined ? _args7[8] : 120000;
                memo = _args7.length > 9 && _args7[9] !== undefined ? _args7[9] : "";
                sequence = _args7.length > 10 && _args7[10] !== undefined ? _args7[10] : null;

                if (fromAddress) {
                  _context7.next = 6;
                  break;
                }

                throw new Error("fromAddress should not be empty");

              case 6:
                if (fundCrypto.checkAddress(fromAddress, _config["default"].BECH32_PREFIX)) {
                  _context7.next = 8;
                  break;
                }

                throw new Error("invalid fromAddress");

              case 8:
                if (!(parseInt(wrkchainId) <= 0)) {
                  _context7.next = 10;
                  break;
                }

                throw new Error("must have wrkchain id");

              case 10:
                if (!(blockHash === "" || blockHash === null || blockHash === undefined)) {
                  _context7.next = 12;
                  break;
                }

                throw new Error("wrkchain must have blockhash");

              case 12:
                if (!(parseInt(height) <= 0)) {
                  _context7.next = 14;
                  break;
                }

                throw new Error("must have height");

              case 14:
                _context7.next = 16;
                return this.getAccount(fromAddress);

              case 16:
                accData = _context7.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context7.next = 19;
                  break;
                }

                throw new Error(accData.message);

              case 19:
                _context7.next = 21;
                return this.getWRKChainParams();

              case 21:
                params = _context7.sent;
                txBody = msgFactory.getMsgRecordWrkChainBlock(wrkchainId, height, blockHash, parentHash, hash1, hash2, hash3, fromAddress);
                authInfo = this.compileAuthInfo(accData.account.sequence, params.params.fee_record, params.params.denom, gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context7.abrupt("return", this.broadcast(signedTxBytes));

              case 26:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function recordWRKChainBlock(_x11, _x12, _x13, _x14, _x15, _x16, _x17) {
        return _recordWRKChainBlock.apply(this, arguments);
      }

      return recordWRKChainBlock;
    }()
    /**
     * Delegate FUND to a validator
     * @param {String} validator
     * @param {Number} amount
     * @param {Object} fee
     * @param {String} denom optional denom
     * @param {String} delegator optional delegator
     * @param {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "delegate",
    value: function () {
      var _delegate = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(validator, amount, fee) {
        var denom,
            delegator,
            memo,
            _convertAmount3,
            amt,
            dnm,
            _convertAmount4,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args8 = arguments;

        return _regenerator["default"].wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                denom = _args8.length > 3 && _args8[3] !== undefined ? _args8[3] : "nund";
                delegator = _args8.length > 4 && _args8[4] !== undefined ? _args8[4] : this.address;
                memo = _args8.length > 5 && _args8[5] !== undefined ? _args8[5] : "";

                if (delegator) {
                  _context8.next = 5;
                  break;
                }

                throw new Error("delegator should not be empty");

              case 5:
                if (fundCrypto.checkAddress(delegator, _config["default"].BECH32_PREFIX)) {
                  _context8.next = 7;
                  break;
                }

                throw new Error("invalid delegator");

              case 7:
                if (validator) {
                  _context8.next = 9;
                  break;
                }

                throw new Error("validator should not be empty");

              case 9:
                if (fundCrypto.checkAddress(validator, _config["default"].BECH32_VAL_PREFIX)) {
                  _context8.next = 11;
                  break;
                }

                throw new Error("invalid validator");

              case 11:
                if (!(amount === 0)) {
                  _context8.next = 13;
                  break;
                }

                throw new Error("amount should not be zero");

              case 13:
                (0, _validateHelper.checkNumber)(amount, "amount");
                _convertAmount3 = (0, _utils.convertAmount)(amount, denom), amt = _convertAmount3.amt, dnm = _convertAmount3.dnm;
                _convertAmount4 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount4.amt, feeDnm = _convertAmount4.dnm;
                _context8.next = 18;
                return this.getAccount(delegator);

              case 18:
                accData = _context8.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context8.next = 21;
                  break;
                }

                throw new Error(accData.message);

              case 21:
                txBody = msgFactory.getMsgDelegate(delegator, validator, amt, dnm, memo);
                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context8.abrupt("return", this.broadcast(signedTxBytes));

              case 25:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function delegate(_x18, _x19, _x20) {
        return _delegate.apply(this, arguments);
      }

      return delegate;
    }()
    /**
     * Undelegate FUND from a validator
     * @param {String} validator
     * @param {Number} amount
     * @param {Object} fee
     * @param {String} denom optional denom
     * @param {String} delegator optional delegator
     * @param {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "undelegate",
    value: function () {
      var _undelegate = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(validator, amount, fee) {
        var denom,
            delegator,
            memo,
            _convertAmount5,
            amt,
            dnm,
            _convertAmount6,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args9 = arguments;

        return _regenerator["default"].wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                denom = _args9.length > 3 && _args9[3] !== undefined ? _args9[3] : "nund";
                delegator = _args9.length > 4 && _args9[4] !== undefined ? _args9[4] : this.address;
                memo = _args9.length > 5 && _args9[5] !== undefined ? _args9[5] : "";

                if (delegator) {
                  _context9.next = 5;
                  break;
                }

                throw new Error("delegator should not be empty");

              case 5:
                if (fundCrypto.checkAddress(delegator, _config["default"].BECH32_PREFIX)) {
                  _context9.next = 7;
                  break;
                }

                throw new Error("invalid delegator");

              case 7:
                if (validator) {
                  _context9.next = 9;
                  break;
                }

                throw new Error("validator should not be empty");

              case 9:
                if (fundCrypto.checkAddress(validator, _config["default"].BECH32_VAL_PREFIX)) {
                  _context9.next = 11;
                  break;
                }

                throw new Error("invalid validator");

              case 11:
                if (!(amount === 0)) {
                  _context9.next = 13;
                  break;
                }

                throw new Error("amount should not be zero");

              case 13:
                (0, _validateHelper.checkNumber)(amount, "amount");
                _convertAmount5 = (0, _utils.convertAmount)(amount, denom), amt = _convertAmount5.amt, dnm = _convertAmount5.dnm;
                _convertAmount6 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount6.amt, feeDnm = _convertAmount6.dnm;
                _context9.next = 18;
                return this.getAccount(delegator);

              case 18:
                accData = _context9.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context9.next = 21;
                  break;
                }

                throw new Error(accData.message);

              case 21:
                txBody = msgFactory.getMsgUndelegate(delegator, validator, amt, dnm, memo);
                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context9.abrupt("return", this.broadcast(signedTxBytes));

              case 25:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function undelegate(_x21, _x22, _x23) {
        return _undelegate.apply(this, arguments);
      }

      return undelegate;
    }()
    /**
     * Redelegate FUND from one validator to another
     * @param {String} validatorFrom
     * @param {String} validatorTo
     * @param {Number} amount
     * @param {Object} fee
     * @param {String} denom optional denom
     * @param {String} delegator optional delegator
     * @param {String} memo optional memo
     * @param {Number} sequence optional sequence
     * @returns {Promise<*>}
     */

  }, {
    key: "redelegate",
    value: function () {
      var _redelegate = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10(validatorFrom, validatorTo, amount, fee) {
        var denom,
            delegator,
            memo,
            _convertAmount7,
            amt,
            dnm,
            _convertAmount8,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args10 = arguments;

        return _regenerator["default"].wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                denom = _args10.length > 4 && _args10[4] !== undefined ? _args10[4] : "nund";
                delegator = _args10.length > 5 && _args10[5] !== undefined ? _args10[5] : this.address;
                memo = _args10.length > 6 && _args10[6] !== undefined ? _args10[6] : "";

                if (delegator) {
                  _context10.next = 5;
                  break;
                }

                throw new Error("delegator should not be empty");

              case 5:
                if (fundCrypto.checkAddress(delegator, _config["default"].BECH32_PREFIX)) {
                  _context10.next = 7;
                  break;
                }

                throw new Error("invalid delegator");

              case 7:
                if (validatorFrom) {
                  _context10.next = 9;
                  break;
                }

                throw new Error("validator should not be empty");

              case 9:
                if (fundCrypto.checkAddress(validatorFrom, _config["default"].BECH32_VAL_PREFIX)) {
                  _context10.next = 11;
                  break;
                }

                throw new Error("invalid validatorFrom");

              case 11:
                if (validatorTo) {
                  _context10.next = 13;
                  break;
                }

                throw new Error("validator should not be empty");

              case 13:
                if (fundCrypto.checkAddress(validatorTo, _config["default"].BECH32_VAL_PREFIX)) {
                  _context10.next = 15;
                  break;
                }

                throw new Error("invalid validatorTo");

              case 15:
                if (!(amount === 0)) {
                  _context10.next = 17;
                  break;
                }

                throw new Error("amount should not be zero");

              case 17:
                (0, _validateHelper.checkNumber)(amount, "amount");
                _convertAmount7 = (0, _utils.convertAmount)(amount, denom), amt = _convertAmount7.amt, dnm = _convertAmount7.dnm;
                _convertAmount8 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount8.amt, feeDnm = _convertAmount8.dnm;
                _context10.next = 22;
                return this.getAccount(delegator);

              case 22:
                accData = _context10.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context10.next = 25;
                  break;
                }

                throw new Error(accData.message);

              case 25:
                txBody = msgFactory.getMsgBeginRedelegate(delegator, validatorFrom, validatorTo, amt, dnm, memo);
                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context10.abrupt("return", this.broadcast(signedTxBytes));

              case 29:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function redelegate(_x24, _x25, _x26, _x27) {
        return _redelegate.apply(this, arguments);
      }

      return redelegate;
    }()
    /**
     * Withdraw Delegator rewards
     * @param {String} validator
     * @param {Object} fee
     * @param {Boolean} commission optional include validator commission
     * @param {String} delegator optional delegator
     * @param {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "withdrawDelegationReward",
    value: function () {
      var _withdrawDelegationReward = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(validator, fee) {
        var commission,
            delegator,
            memo,
            _convertAmount9,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args11 = arguments;

        return _regenerator["default"].wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                commission = _args11.length > 2 && _args11[2] !== undefined ? _args11[2] : false;
                delegator = _args11.length > 3 && _args11[3] !== undefined ? _args11[3] : this.address;
                memo = _args11.length > 4 && _args11[4] !== undefined ? _args11[4] : "";

                if (delegator) {
                  _context11.next = 5;
                  break;
                }

                throw new Error("delegator should not be empty");

              case 5:
                if (fundCrypto.checkAddress(delegator, _config["default"].BECH32_PREFIX)) {
                  _context11.next = 7;
                  break;
                }

                throw new Error("invalid delegator");

              case 7:
                if (validator) {
                  _context11.next = 9;
                  break;
                }

                throw new Error("validator should not be empty");

              case 9:
                if (!commission) {
                  _context11.next = 12;
                  break;
                }

                if ((0, _validateHelper.isSelfDelegator)(validator, delegator, _config["default"].BECH32_PREFIX)) {
                  _context11.next = 12;
                  break;
                }

                throw new Error("delegator is not the delf delegator address");

              case 12:
                _convertAmount9 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount9.amt, feeDnm = _convertAmount9.dnm;
                _context11.next = 15;
                return this.getAccount(delegator);

              case 15:
                accData = _context11.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context11.next = 18;
                  break;
                }

                throw new Error(accData.message);

              case 18:
                txBody = msgFactory.getMsgWithdrawDelegatorReward(delegator, validator, commission, memo);
                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context11.abrupt("return", this.broadcast(signedTxBytes));

              case 22:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      function withdrawDelegationReward(_x28, _x29) {
        return _withdrawDelegationReward.apply(this, arguments);
      }

      return withdrawDelegationReward;
    }()
    /**
     * Vote on a Governance Proposal
     * @param {Number} proposalId
     * @param {String} option yes | no | abstain | nowithveto
     * @param {Object} fee
     * @param {String} voter optional voter
     * @param {String} memo optional memo
     * @returns {Promise<*>}
     */

  }, {
    key: "voteOnProposal",
    value: function () {
      var _voteOnProposal = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee12(proposalId, option, fee) {
        var voter,
            memo,
            _convertAmount10,
            feeAmt,
            feeDnm,
            accData,
            txBody,
            authInfo,
            signedTxBytes,
            _args12 = arguments;

        return _regenerator["default"].wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                voter = _args12.length > 3 && _args12[3] !== undefined ? _args12[3] : this.address;
                memo = _args12.length > 4 && _args12[4] !== undefined ? _args12[4] : "";

                if (voter) {
                  _context12.next = 4;
                  break;
                }

                throw new Error("voter should not be empty");

              case 4:
                if (fundCrypto.checkAddress(voter, _config["default"].BECH32_PREFIX)) {
                  _context12.next = 6;
                  break;
                }

                throw new Error("invalid voter");

              case 6:
                _context12.t0 = option;
                _context12.next = _context12.t0 === "VOTE_OPTION_YES" ? 9 : _context12.t0 === "VOTE_OPTION_ABSTAIN" ? 9 : _context12.t0 === "VOTE_OPTION_NO" ? 9 : _context12.t0 === "VOTE_OPTION_NO_WITH_VETO" ? 9 : 10;
                break;

              case 9:
                return _context12.abrupt("break", 11);

              case 10:
                throw new Error("\"".concat(option, "\" is an invalid vote option"));

              case 11:
                _convertAmount10 = (0, _utils.convertAmount)(fee.amount, fee.denom), feeAmt = _convertAmount10.amt, feeDnm = _convertAmount10.dnm;
                _context12.next = 14;
                return this.getAccount(voter);

              case 14:
                accData = _context12.sent;

                if (!(accData !== null && accData !== void 0 && accData.code)) {
                  _context12.next = 17;
                  break;
                }

                throw new Error(accData.message);

              case 17:
                txBody = msgFactory.getMsgVote(proposalId, option, voter, memo);
                authInfo = this.compileAuthInfo(accData.account.sequence, feeAmt, feeDnm, fee.gas); // -------------------------------- sign --------------------------------

                signedTxBytes = this.sign(txBody, authInfo);
                return _context12.abrupt("return", this.broadcast(signedTxBytes));

              case 21:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      function voteOnProposal(_x30, _x31, _x32) {
        return _voteOnProposal.apply(this, arguments);
      }

      return voteOnProposal;
    }()
    /**
     * generates the required auth info to be included with the tx body for signing
     * @param sequence {Number} account sequence
     * @param feeAmount {Number} fee amount
     * @param feeDenom {String} fee denomination
     * @param gas {Number} gas limit
     * @param pubKeyAny
     * @returns {*}
     */

  }, {
    key: "compileAuthInfo",
    value: function compileAuthInfo(sequence, feeAmount, feeDenom, gas) {
      var pubKeyAny = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : this.pubKeyAny;
      var signerInfo = msgFactory.getSignerInfo(pubKeyAny, sequence);
      var feeValue = msgFactory.getFee(feeAmount, feeDenom, gas);
      return msgFactory.getAuthInfo(signerInfo, feeValue);
    }
    /*
     * API getters
     */

    /**
     * get chain & network info
     * @returns {Promise<{result: {error: *}, status: number}|{result: *, status: *}|void>}
     */

  }, {
    key: "getChainInfo",
    value: function getChainInfo() {
      var nodeInfo = "/node_info";
      return (0, _nodeFetch["default"])(this.url + nodeInfo).then(function (response) {
        return response.json();
      });
    }
    /**
     * get account info
     * @param {String} address optional address
     * @returns {Promise<{result: {error: *}, status: number}|{result: *, status: *}|void>}
     */

  }, {
    key: "getAccount",
    value: function getAccount() {
      var address = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.address;
      var api = "/cosmos/auth/v1beta1/accounts/";
      return (0, _nodeFetch["default"])(this.url + api + address).then(function (response) {
        return response.json();
      });
    }
    /**
     * get BEACON params
     * @returns {Promise<{result: {error: *}, status: number}|{result: *, status: *}|void>}
     */

  }, {
    key: "getBeaconParams",
    value: function () {
      var _getBeaconParams = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee13() {
        var api;
        return _regenerator["default"].wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                api = "/mainchain/beacon/v1/params";
                return _context13.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function getBeaconParams() {
        return _getBeaconParams.apply(this, arguments);
      }

      return getBeaconParams;
    }()
    /**
     * get WRKChain params
     * @returns {Promise<{result: {error: *}, status: number}|{result: *, status: *}|void>}
     */

  }, {
    key: "getWRKChainParams",
    value: function () {
      var _getWRKChainParams = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee14() {
        var api;
        return _regenerator["default"].wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                api = "/mainchain/wrkchain/v1/params";
                return _context14.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      function getWRKChainParams() {
        return _getWRKChainParams.apply(this, arguments);
      }

      return getWRKChainParams;
    }()
    /**
     * get balances for given address
     * @param {String} address optional address
     * @param {String} denom optional denomination
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getBalance",
    value: function () {
      var _getBalance = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee15() {
        var address,
            denom,
            api,
            res,
            _args15 = arguments;
        return _regenerator["default"].wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                address = _args15.length > 0 && _args15[0] !== undefined ? _args15[0] : this.address;
                denom = _args15.length > 1 && _args15[1] !== undefined ? _args15[1] : "nund";
                api = "/cosmos/bank/v1beta1/balances/".concat(address);
                _context15.next = 5;
                return (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                });

              case 5:
                res = _context15.sent;
                return _context15.abrupt("return", (res === null || res === void 0 ? void 0 : res.balances) || []);

              case 7:
              case "end":
                return _context15.stop();
            }
          }
        }, _callee15, this);
      }));

      function getBalance() {
        return _getBalance.apply(this, arguments);
      }

      return getBalance;
    }()
    /**
     * get transactions for an account
     * @param {String} address optional address
     * @param {Number} page page number, default 1
     * @param {Number} limit number of results per page, default 100, max 100
     * @param {String} order optional order
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getTransactions",
    value: function () {
      var _getTransactions = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee16() {
        var address,
            page,
            limit,
            order,
            api,
            _args16 = arguments;
        return _regenerator["default"].wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                address = _args16.length > 0 && _args16[0] !== undefined ? _args16[0] : this.address;
                page = _args16.length > 1 && _args16[1] !== undefined ? _args16[1] : 1;
                limit = _args16.length > 2 && _args16[2] !== undefined ? _args16[2] : 100;
                order = _args16.length > 3 && _args16[3] !== undefined ? _args16[3] : "ORDER_BY_DESC";
                if (limit > 100) limit = 100;
                api = "/cosmos/tx/v1beta1/txs?events=message.sender%3D'".concat(address, "'&pagination.limit=").concat(limit, "&order_by=").concat(order, "&pagination.count_total=true&pagination.offset=").concat(page - 1);
                return _context16.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 7:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      function getTransactions() {
        return _getTransactions.apply(this, arguments);
      }

      return getTransactions;
    }()
    /**
     * get transactions received by an account - specifically, FUND transfers sent to the address
     * @param {String} address optional address
     * @param {Number} page page number, default 1
     * @param {Number} limit number of results per page, default 100, max 100
     * @param {String} order optional order
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getTransactionsReceived",
    value: function () {
      var _getTransactionsReceived = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee17() {
        var address,
            page,
            limit,
            order,
            api,
            _args17 = arguments;
        return _regenerator["default"].wrap(function _callee17$(_context17) {
          while (1) {
            switch (_context17.prev = _context17.next) {
              case 0:
                address = _args17.length > 0 && _args17[0] !== undefined ? _args17[0] : this.address;
                page = _args17.length > 1 && _args17[1] !== undefined ? _args17[1] : 1;
                limit = _args17.length > 2 && _args17[2] !== undefined ? _args17[2] : 100;
                order = _args17.length > 3 && _args17[3] !== undefined ? _args17[3] : "ORDER_BY_DESC";
                if (limit > 100) limit = 100;
                api = "/cosmos/tx/v1beta1/txs?events=transfer.recipient%3D'".concat(address, "'&pagination.limit=").concat(limit, "&order_by=").concat(order, "&pagination.count_total=true&pagination.offset=").concat(page - 1);
                return _context17.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 7:
              case "end":
                return _context17.stop();
            }
          }
        }, _callee17, this);
      }));

      function getTransactionsReceived() {
        return _getTransactionsReceived.apply(this, arguments);
      }

      return getTransactionsReceived;
    }()
    /**
     * get governance proposals
     * @param {Number} page page number, default 1
     * @param {Number} limit number of results per page, default 100, max 100
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getGovernanceProposals",
    value: function () {
      var _getGovernanceProposals = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee18() {
        var page,
            limit,
            api,
            _args18 = arguments;
        return _regenerator["default"].wrap(function _callee18$(_context18) {
          while (1) {
            switch (_context18.prev = _context18.next) {
              case 0:
                page = _args18.length > 0 && _args18[0] !== undefined ? _args18[0] : 1;
                limit = _args18.length > 1 && _args18[1] !== undefined ? _args18[1] : 100;
                if (limit > 100) limit = 100;
                api = "/cosmos/gov/v1beta1/proposals?pagination.limit=".concat(limit, "&pagination.count_total=true&pagination.offset=").concat(page - 1);
                return _context18.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 5:
              case "end":
                return _context18.stop();
            }
          }
        }, _callee18, this);
      }));

      function getGovernanceProposals() {
        return _getGovernanceProposals.apply(this, arguments);
      }

      return getGovernanceProposals;
    }()
    /**
     * get governance proposals votes
     * @param {Number} proposalId id of proposal
     * @param {Number} page page number, default 1
     * @param {Number} limit number of results per page, default 100, max 100
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getGovernanceProposalVotes",
    value: function () {
      var _getGovernanceProposalVotes = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee19(proposalId) {
        var page,
            limit,
            api,
            _args19 = arguments;
        return _regenerator["default"].wrap(function _callee19$(_context19) {
          while (1) {
            switch (_context19.prev = _context19.next) {
              case 0:
                page = _args19.length > 1 && _args19[1] !== undefined ? _args19[1] : 1;
                limit = _args19.length > 2 && _args19[2] !== undefined ? _args19[2] : 100;
                if (limit > 100) limit = 100;
                api = "/cosmos/gov/v1beta1/proposals/".concat(proposalId, "/votes?pagination.limit=").concat(limit, "&pagination.count_total=true&pagination.offset=").concat(page - 1);
                return _context19.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 5:
              case "end":
                return _context19.stop();
            }
          }
        }, _callee19, this);
      }));

      function getGovernanceProposalVotes(_x33) {
        return _getGovernanceProposalVotes.apply(this, arguments);
      }

      return getGovernanceProposalVotes;
    }()
    /**
     * get governance proposals votes
     * @param {Number} proposalId id of proposal
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getGovernanceProposalTally",
    value: function () {
      var _getGovernanceProposalTally = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee20(proposalId) {
        var api;
        return _regenerator["default"].wrap(function _callee20$(_context20) {
          while (1) {
            switch (_context20.prev = _context20.next) {
              case 0:
                api = "/cosmos/gov/v1beta1/proposals/".concat(proposalId, "/tally");
                return _context20.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context20.stop();
            }
          }
        }, _callee20, this);
      }));

      function getGovernanceProposalTally(_x34) {
        return _getGovernanceProposalTally.apply(this, arguments);
      }

      return getGovernanceProposalTally;
    }()
    /**
     * get transaction
     * @param {String} hash the transaction hash
     * @return {Promise} resolves with http response
     */

  }, {
    key: "getTx",
    value: function () {
      var _getTx = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee21(hash) {
        var api;
        return _regenerator["default"].wrap(function _callee21$(_context21) {
          while (1) {
            switch (_context21.prev = _context21.next) {
              case 0:
                api = "/cosmos/tx/v1beta1/txs/".concat(hash);
                return _context21.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context21.stop();
            }
          }
        }, _callee21, this);
      }));

      function getTx(_x35) {
        return _getTx.apply(this, arguments);
      }

      return getTx;
    }()
    /**
     * get delegations for address
     * @param {String} delegator optional address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getDelegations",
    value: function () {
      var _getDelegations = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee22() {
        var delegator,
            api,
            _args22 = arguments;
        return _regenerator["default"].wrap(function _callee22$(_context22) {
          while (1) {
            switch (_context22.prev = _context22.next) {
              case 0:
                delegator = _args22.length > 0 && _args22[0] !== undefined ? _args22[0] : this.address;
                api = "/cosmos/staking/v1beta1/delegations/".concat(delegator);
                return _context22.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 3:
              case "end":
                return _context22.stop();
            }
          }
        }, _callee22, this);
      }));

      function getDelegations() {
        return _getDelegations.apply(this, arguments);
      }

      return getDelegations;
    }()
    /**
     * get unbonding delegations for address
     * @param {String} delegator optional Bech32 address
     * @param {String} valAddress optional Bech32 operator address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getUnbondingDelegations",
    value: function () {
      var _getUnbondingDelegations = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee23() {
        var delegator,
            valAddress,
            api,
            _args23 = arguments;
        return _regenerator["default"].wrap(function _callee23$(_context23) {
          while (1) {
            switch (_context23.prev = _context23.next) {
              case 0:
                delegator = _args23.length > 0 && _args23[0] !== undefined ? _args23[0] : this.address;
                valAddress = _args23.length > 1 && _args23[1] !== undefined ? _args23[1] : "";
                api = "/cosmos/staking/v1beta1/delegators/".concat(delegator, "/unbonding_delegations");
                return _context23.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 4:
              case "end":
                return _context23.stop();
            }
          }
        }, _callee23, this);
      }));

      function getUnbondingDelegations() {
        return _getUnbondingDelegations.apply(this, arguments);
      }

      return getUnbondingDelegations;
    }()
    /**
     * get delegator address's rewards
     * @param {String} delegator optional address
     * @param {String} valAddress optional Bech32 operator address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getDelegatorRewards",
    value: function () {
      var _getDelegatorRewards = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee24() {
        var delegator,
            valAddress,
            suffix,
            api,
            _args24 = arguments;
        return _regenerator["default"].wrap(function _callee24$(_context24) {
          while (1) {
            switch (_context24.prev = _context24.next) {
              case 0:
                delegator = _args24.length > 0 && _args24[0] !== undefined ? _args24[0] : this.address;
                valAddress = _args24.length > 1 && _args24[1] !== undefined ? _args24[1] : "";
                suffix = "";

                if (valAddress.length > 0) {
                  suffix = "/".concat(valAddress);
                }

                api = "/cosmos/distribution/v1beta1/delegators/".concat(delegator, "/rewards").concat(suffix);
                return _context24.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 6:
              case "end":
                return _context24.stop();
            }
          }
        }, _callee24, this);
      }));

      function getDelegatorRewards() {
        return _getDelegatorRewards.apply(this, arguments);
      }

      return getDelegatorRewards;
    }()
    /**
     * get delegator's current withdraw address
     * @param {String} delegator optional address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getDelegatorWithdrawAddress",
    value: function () {
      var _getDelegatorWithdrawAddress = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee25() {
        var delegator,
            api,
            _args25 = arguments;
        return _regenerator["default"].wrap(function _callee25$(_context25) {
          while (1) {
            switch (_context25.prev = _context25.next) {
              case 0:
                delegator = _args25.length > 0 && _args25[0] !== undefined ? _args25[0] : this.address;
                api = "/cosmos/distribution/v1beta1/delegators/".concat(delegator, "/withdraw_address");
                return _context25.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 3:
              case "end":
                return _context25.stop();
            }
          }
        }, _callee25, this);
      }));

      function getDelegatorWithdrawAddress() {
        return _getDelegatorWithdrawAddress.apply(this, arguments);
      }

      return getDelegatorWithdrawAddress;
    }()
    /**
     * get a list of current validators based on filters
     * @param {String} status optional status. one of BOND_STATUS_BONDED, BOND_STATUS_UNBONDED, BOND_STATUS_UNBONDING. Default BOND_STATUS_BONDED
     * @param {Number} page optional page
     * @param {Number} limit optional limit
     * @param {String} valAddress optional Bech32 operator address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getValidators",
    value: function () {
      var _getValidators = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee26() {
        var status,
            page,
            limit,
            valAddress,
            suffix,
            api,
            _args26 = arguments;
        return _regenerator["default"].wrap(function _callee26$(_context26) {
          while (1) {
            switch (_context26.prev = _context26.next) {
              case 0:
                status = _args26.length > 0 && _args26[0] !== undefined ? _args26[0] : "BOND_STATUS_BONDED";
                page = _args26.length > 1 && _args26[1] !== undefined ? _args26[1] : 1;
                limit = _args26.length > 2 && _args26[2] !== undefined ? _args26[2] : 100;
                valAddress = _args26.length > 3 && _args26[3] !== undefined ? _args26[3] : "";
                if (limit > 100) limit = 100;
                _context26.t0 = status;
                _context26.next = _context26.t0 === "BOND_STATUS_BONDED" ? 8 : _context26.t0 === "BOND_STATUS_UNBONDED" ? 8 : _context26.t0 === "BOND_STATUS_UNBONDING" ? 8 : 9;
                break;

              case 8:
                return _context26.abrupt("break", 11);

              case 9:
                status = "BOND_STATUS_BONDED";
                return _context26.abrupt("break", 11);

              case 11:
                suffix = "?status=".concat(status, "&pagination.offset=").concat(page - 1, "&pagination.limit=").concat(limit);

                if (valAddress.length > 0) {
                  suffix = "/".concat(valAddress);
                }

                api = "/cosmos/staking/v1beta1/validators".concat(suffix);
                return _context26.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 15:
              case "end":
                return _context26.stop();
            }
          }
        }, _callee26, this);
      }));

      function getValidators() {
        return _getValidators.apply(this, arguments);
      }

      return getValidators;
    }()
    /**
     * get redelegations, with optional filters
     * @param {String} delegator optional delAddress Bech32 address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getRedelegations",
    value: function () {
      var _getRedelegations = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee27() {
        var delegator,
            api,
            _args27 = arguments;
        return _regenerator["default"].wrap(function _callee27$(_context27) {
          while (1) {
            switch (_context27.prev = _context27.next) {
              case 0:
                delegator = _args27.length > 0 && _args27[0] !== undefined ? _args27[0] : this.address;
                api = "/cosmos/staking/v1beta1/delegators/".concat(delegator, "/redelegations");
                return _context27.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 3:
              case "end":
                return _context27.stop();
            }
          }
        }, _callee27, this);
      }));

      function getRedelegations() {
        return _getRedelegations.apply(this, arguments);
      }

      return getRedelegations;
    }()
    /**
     * get commission accrued by a validator
     * @param {String} delegator optional delAddress Bech32 address
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getValidatorCommission",
    value: function () {
      var _getValidatorCommission = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee28(validator) {
        var api;
        return _regenerator["default"].wrap(function _callee28$(_context28) {
          while (1) {
            switch (_context28.prev = _context28.next) {
              case 0:
                api = "/cosmos/distribution/v1beta1/validators/".concat(validator, "/commission");
                return _context28.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context28.stop();
            }
          }
        }, _callee28, this);
      }));

      function getValidatorCommission(_x36) {
        return _getValidatorCommission.apply(this, arguments);
      }

      return getValidatorCommission;
    }()
    /**
     * get total supply of FUND
     * @returns {Promise} resolves with http response
     */

  }, {
    key: "getTotalSupply",
    value: function () {
      var _getTotalSupply = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee29() {
        var api;
        return _regenerator["default"].wrap(function _callee29$(_context29) {
          while (1) {
            switch (_context29.prev = _context29.next) {
              case 0:
                api = "/mainchain/enterprise/v1/supply";
                return _context29.abrupt("return", (0, _nodeFetch["default"])(this.url + api).then(function (response) {
                  return response.json();
                }));

              case 2:
              case "end":
                return _context29.stop();
            }
          }
        }, _callee29, this);
      }));

      function getTotalSupply() {
        return _getTotalSupply.apply(this, arguments);
      }

      return getTotalSupply;
    }()
    /*
     * Sign & Broadcast functions
     */

  }, {
    key: "sign",
    value: function sign(txBody, authInfo) {
      var accountNumber = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.accountNumber;
      var privKey = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.privKey;

      var bodyBytes = _proto["default"].cosmos.tx.v1beta1.TxBody.encode(txBody).finish();

      var authInfoBytes = _proto["default"].cosmos.tx.v1beta1.AuthInfo.encode(authInfo).finish();

      var signDoc = new _proto["default"].cosmos.tx.v1beta1.SignDoc({
        body_bytes: bodyBytes,
        auth_info_bytes: authInfoBytes,
        chain_id: this.chainId,
        account_number: Number(accountNumber)
      });

      var signMessage = _proto["default"].cosmos.tx.v1beta1.SignDoc.encode(signDoc).finish();

      var hash = _crypto["default"].createHash("sha256").update(signMessage).digest();

      var sig = _secp256k["default"].sign(hash, Buffer.from(privKey));

      var txRaw = new _proto["default"].cosmos.tx.v1beta1.TxRaw({
        body_bytes: bodyBytes,
        auth_info_bytes: authInfoBytes,
        signatures: [sig.signature]
      });

      var txBytes = _proto["default"].cosmos.tx.v1beta1.TxRaw.encode(txRaw).finish();

      var txBytesBase64 = Buffer.from(txBytes, 'binary').toString('base64');
      return txBytes;
    } // "BROADCAST_MODE_UNSPECIFIED", "BROADCAST_MODE_BLOCK", "BROADCAST_MODE_SYNC", "BROADCAST_MODE_ASYNC"

  }, {
    key: "broadcast",
    value: function broadcast(signedTxBytes) {
      var broadCastMode = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.broadcastMode;
      var txBytesBase64 = Buffer.from(signedTxBytes, 'binary').toString('base64');
      var options = {
        method: 'POST',
        url: this.url + '/cosmos/tx/v1beta1/txs',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {
          tx_bytes: txBytesBase64,
          mode: broadCastMode
        },
        json: true
      };
      return new Promise(function (resolve, reject) {
        (0, _request["default"])(options, function (error, response, body) {
          if (error) return reject(error);

          try {
            resolve(body);
          } catch (e) {
            reject(e);
          }
        });
      });
    }
    /*
     * Crypto/wallet interfaces
     */

    /**
     * Creates a private key and returns it and its address.
     * @return {object} the private key and address in an object.
     * {
     *  address,
     *  privateKey
     * }
     */

  }, {
    key: "createAccount",
    value: function createAccount() {
      var privateKey = fundCrypto.generatePrivateKey();
      return {
        privateKey: privateKey,
        address: fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX)
      };
    }
    /**
     * Creates an account keystore object, and returns the private key and address.
     * @param {String} password
     *  {
     *  privateKey,
     *  address,
     *  keystore
     * }
     */

  }, {
    key: "createAccountWithKeystore",
    value: function createAccountWithKeystore(password) {
      if (!password) {
        throw new Error("password should not be falsy");
      }

      var privateKey = fundCrypto.generatePrivateKey();
      var address = fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX);
      var keystore = fundCrypto.generateKeyStore(privateKey, password);
      return {
        privateKey: privateKey,
        address: address,
        keystore: keystore
      };
    }
    /**
     * Creates an account from mnemonic seed phrase.
     * @return {object}
     * {
     *  privateKey,
     *  address,
     *  mnemonic
     * }
     */

  }, {
    key: "createAccountWithMneomnic",
    value: function createAccountWithMneomnic() {
      var mnemonic = fundCrypto.generateMnemonic();
      var privateKey = fundCrypto.getPrivateKeyFromMnemonic(mnemonic);
      var address = fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX);
      return {
        privateKey: privateKey,
        address: address,
        mnemonic: mnemonic
      };
    }
    /**
     * Recovers an account from a keystore object.
     * @param {object} keystore object.
     * @param {string} password password.
     * {
     * privateKey,
     * address
     * }
     */

  }, {
    key: "recoverAccountFromKeystore",
    value: function recoverAccountFromKeystore(keystore, password) {
      var privateKey = fundCrypto.getPrivateKeyFromKeyStore(keystore, password);
      var address = fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX);
      return {
        privateKey: privateKey,
        address: address
      };
    }
    /**
     * Recovers an account from a mnemonic seed phrase.
     * @param {String} mneomnic
     * {
     * privateKey,
     * address
     * }
     */

  }, {
    key: "recoverAccountFromMnemonic",
    value: function recoverAccountFromMnemonic(mnemonic) {
      var privateKey = fundCrypto.getPrivateKeyFromMnemonic(mnemonic);
      var address = fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX);
      return {
        privateKey: privateKey,
        address: address
      };
    }
    /**
     * Recovers an account using private key.
     * @param {String} privateKey
     * {
     * privateKey,
     * address
     * }
     */

  }, {
    key: "recoverAccountFromPrivateKey",
    value: function recoverAccountFromPrivateKey(privateKey) {
      var address = fundCrypto.getAddressFromPrivateKey(privateKey, _config["default"].BECH32_PREFIX);
      return {
        privateKey: privateKey,
        address: address
      };
    }
    /**
     * Validates an address.
     * @param {String} address
     * @param {String} prefix
     * @return {Boolean}
     */

  }, {
    key: "checkAddress",
    value: function checkAddress(address) {
      var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _config["default"].BECH32_PREFIX;
      return fundCrypto.checkAddress(address, prefix);
    }
    /**
     * Returns the address for the current account if setPrivateKey has been called on this client.
     * @return {String}
     */

  }, {
    key: "getClientKeyAddress",
    value: function getClientKeyAddress() {
      if (!this.privateKey) throw new Error("no private key is set on this client");
      var address = fundCrypto.getAddressFromPrivateKey(this.privateKey, _config["default"].BECH32_PREFIX);
      this.address = address;
      return address;
    }
    /**
     * Retuns a standard error mimicking the object returned by _httpClient.request
     * @param {String} errMsg the message to be put in result.error
     * @param {Number} status - status code, optional. Default 400
     * @returns {{result: {error: *}, status: number}}
     * @private
     */

  }, {
    key: "_stdError",
    value: function _stdError(errMsg) {
      var status = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 400;
      var stdError = {
        status: status,
        result: {
          error: errMsg
        }
      };
      return stdError;
    }
  }]);
  return UndClient;
}();

exports.UndClient = UndClient;